# Generated by Django 2.2.3 on 2019-09-27 13:39

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("response", "0010_incident_report_only"),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name="externaluser", unique_together={("owner", "app_id", "external_id")}
        )
    ]
