# Generated by Django 2.2.3 on 2019-09-06 14:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("response", "0004_auto_20190904_1704")]

    operations = [
        migrations.AlterField(
            model_name="timelineevent",
            name="event_type",
            field=models.CharField(help_text="Type of event.", max_length=30),
        )
    ]
