# Generated by Django 2.2.3 on 2019-09-10 14:46

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("response", "0006_externaluser_full_name")]

    operations = [migrations.DeleteModel(name="IncidentExtension")]
