# Generated by Django 2.2.3 on 2019-09-24 15:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("response", "0009_commschannel_channel_name")]

    operations = [
        migrations.AddField(
            model_name="incident",
            name="report_only",
            field=models.BooleanField(default=False),
        )
    ]
