# Generated by Django 2.2.3 on 2019-11-07 17:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("response", "0013_incident_private")]

    operations = [
        migrations.CreateModel(
            name="Event",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("timestamp", models.DateTimeField()),
                ("event_type", models.CharField(max_length=50)),
                ("payload", models.TextField()),
            ],
        )
    ]
